#include "models.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sql.h"

klist_user *klist_user_init() {
    klist_user *user = malloc(sizeof(klist_user));
    user->id = -1;
    user->name = NULL;
    user->local_id = -1;
    user->discord_id = -1;
    user->google_id = -1;
    return user;
}

klist_user *klist_user_init_from_sql(sqlite3_stmt *stmt) {
    klist_user *user = klist_user_init();
    size_t bytes;
    user->id = sqlite3_column_int(stmt, 0);
    if ((bytes = sqlite3_column_bytes(stmt, 1)) > 0) {
        user->name = malloc(bytes);
        memcpy(user->name, sqlite3_column_text(stmt, 1), bytes);
    }
    if (sqlite3_column_type(stmt, 2) != SQLITE_NULL) user->local_id = sqlite3_column_int(stmt, 2);
    if (sqlite3_column_type(stmt, 3) != SQLITE_NULL) user->discord_id = sqlite3_column_int(stmt, 3);
    if (sqlite3_column_type(stmt, 4) != SQLITE_NULL) user->google_id = sqlite3_column_int(stmt, 4);
    return user;
}

klist_user *klist_user_get_by_id(klist *ctx, u_int id) {
    sqlite3_stmt *stmt = ctx->stmts[GET_USER];
    sqlite3_bind_int(stmt, 1, id);
    if (sqlite3_step(stmt) == SQLITE_ROW) {
        sqlite3_reset(stmt);
        return klist_user_init_from_sql(stmt);
    }
    return NULL;
}

klist_user *klist_user_get_by_local(klist *ctx, u_int local_id) {
    sqlite3_stmt *stmt = ctx->stmts[GET_USER_BY_LOCAL];
    sqlite3_bind_int(stmt, 1, local_id);
    if (sqlite3_step(stmt) == SQLITE_ROW) {
        sqlite3_reset(stmt);
        return klist_user_init_from_sql(stmt);
    }
    return NULL;
}

void klist_user_save(klist *ctx, klist_user *user) {
    sqlite3_stmt *stmt = ctx->stmts[ADD_USER];
    int result = 0;
    sqlite3_bind_text(stmt, 1, (char*)user->name, -1, SQLITE_STATIC);
    sqlite3_bind_int(stmt, 2, user->local_id);
    sqlite3_bind_int(stmt, 3, user->discord_id > 0 ? user->discord_id : -1);
    sqlite3_bind_int(stmt, 4, user->google_id > 0 ? user->google_id : -1);
    if ((result = sqlite3_step(stmt)) == SQLITE_ROW) {
        user->id = sqlite3_column_int(stmt, 1);
        sqlite3_reset(stmt);
    } else fprintf(stderr, "failed to save user: %s - %d\n", sqlite3_errmsg(ctx->db), result);
}

void klist_user_delete(klist *ctx, klist_user *user) {
    sqlite3_stmt *stmt = ctx->stmts[DEL_USER];
    sqlite3_bind_int(stmt, 1, (int)user->id);
    if (sqlite3_step(stmt) == SQLITE_ROW)
        sqlite3_reset(stmt);
}

void klist_user_deinit(klist_user *user) {
    free(user->name);
    free(user);
}


klist_stage *klist_stage_init() {
    klist_stage *stage = malloc(sizeof(klist_stage));
    stage->id = -1;
    stage->name = NULL;
    stage->desc = NULL;
    stage->list_id = -1;
    return stage;
}

klist_stage *klist_stage_init_from_sql(sqlite3_stmt *stmt) {
    klist_stage *stage = klist_stage_init();
    size_t bytes;
    stage->id = sqlite3_column_int(stmt, 0);
    if ((bytes = sqlite3_column_bytes(stmt, 1)) > 0) {
        stage->name = malloc(bytes);
        memcpy(stage->name, sqlite3_column_text(stmt, 1), bytes);
    }
    if ((bytes = sqlite3_column_bytes(stmt, 2)) > 0) {
        stage->desc = malloc(bytes);
        memcpy(stage->desc, sqlite3_column_text(stmt, 2), bytes);
    }
    return stage;
}

klist_stage *klist_stage_get_by_id(klist *ctx, u_int id) {
    sqlite3_stmt *stmt = ctx->stmts[GET_STAGE];
    sqlite3_bind_int(stmt, 1, id);
    if (sqlite3_step(stmt) == SQLITE_ROW) {
        sqlite3_reset(stmt);
        return klist_stage_init_from_sql(stmt);
    }
    return NULL;
}

void klist_stage_deinit(klist_stage *stage) {
    if (stage->name) free(stage->name);
    if (stage->desc) free(stage->desc);
    free(stage);
}


klist_list *klist_list_init() {
    klist_list *list = malloc(sizeof(klist_list));
    list->id = -1;
    list->name = NULL;
    list->desc = NULL;
    return list;
}

klist_list *klist_list_init_from_sql(sqlite3_stmt *stmt) {
    klist_list *list = klist_list_init();
    size_t bytes;
    list->id = sqlite3_column_int(stmt, 0);
    if ((bytes = sqlite3_column_bytes(stmt, 1)) > 0) {
        list->name = malloc(bytes);
        memcpy(list->name, sqlite3_column_text(stmt, 1), bytes);
    }
    if ((bytes = sqlite3_column_bytes(stmt, 2)) > 0) {
        list->desc = malloc(bytes);
        memcpy(list->desc, sqlite3_column_text(stmt, 2), bytes);
    }
    return list;
}

klist_list *klist_list_get_by_id(klist *ctx, u_int id) {
    sqlite3_stmt *stmt = ctx->stmts[GET_LIST];
    sqlite3_bind_int(stmt, 1, id);
    if (sqlite3_step(stmt) == SQLITE_ROW) {
        sqlite3_reset(stmt);
        return klist_list_init_from_sql(stmt);
    }
    return NULL;
}

klist_list **klist_list_get_all_by_user(klist *ctx, u_int user_id, size_t *count) {
    sqlite3_stmt *stmt = ctx->stmts[GET_LISTS_BY_USER];
    sqlite3_bind_int(stmt, 1, user_id);
    klist_list **lists = NULL;
    size_t lists_len = 0;
    while (sqlite3_step(stmt) == SQLITE_ROW) {
        lists_len++;
        lists = realloc(lists, lists_len * sizeof(klist_list *));
        lists[lists_len - 1] = klist_list_init(stmt);
    }
    if (count != NULL) *count = lists_len;
    return lists;
}

void klist_list_deinit(klist_list *list) {
    if (list->name) free(list->name);
    if (list->desc) free(list->desc);
    free(list);
}


klist_task *klist_task_init() {
    klist_task *task = malloc(sizeof(klist_task));
    task->id = -1;
    task->name = NULL;
    task->desc = NULL;
    return task;
}

klist_task *klist_task_init_from_sql(sqlite3_stmt *stmt) {
    klist_task *task = klist_task_init();
    size_t bytes;
    task->id = sqlite3_column_int(stmt, 0);
    if ((bytes = sqlite3_column_bytes(stmt, 1)) > 0) {
        task->name = malloc(bytes);
        memcpy(task->name, sqlite3_column_text(stmt, 1), bytes);
    }
    if ((bytes = sqlite3_column_bytes(stmt, 2)) > 0) {
        task->desc = malloc(bytes);
        memcpy(task->desc, sqlite3_column_text(stmt, 2), bytes);
    }
    return task;
}

klist_task *klist_task_get_by_id(klist *ctx, u_int id) {
    sqlite3_stmt *stmt = ctx->stmts[GET_TASK];
    sqlite3_bind_int(stmt, 1, id);
    if (sqlite3_step(stmt) == SQLITE_ROW) {
        sqlite3_reset(stmt);
        return klist_task_init_from_sql(stmt);
    }
    return NULL;
}

void klist_task_deinit(klist_task *task) {
    if (task->name) free(task->name);
    if (task->desc) free(task->desc);
    free(task);
}
